'user strict';

var xSelector = function (selector) {
    return {
        selector: selector,
        locateStrategy: 'xpath'
    }
};

var preferenceDialogueCommand= {

    openDialog: function(callback) {
        this.api.pause(1000);

        return this.waitForElementVisible('@preferenceManagerBtn', 1000)
            .click('@preferenceManagerBtn', callback);
    },
    clickResetButton: function() {
        this.api.pause(1000);

        return this.waitForElementVisible('@resetBtn', 1000)
            .click('@resetBtn');
    },
    clickPackagesButton: function(callback) {
        this.api.pause(1000);
        return this.click('@packagesBtn').waitForElementVisible('@packagesDialog', 1000, callback);
    },
    clickDisplayButton: function(callback) {

        return this.click('@displayBtn', callback);
    },
    setPackagesHarkValue: function(callback) {
        return this.setValue('@packagesDialogValue6', 'Sea', callback);
    },
    setPackagesEffectToDarkValue: function (callback) {
        return this.setValue('@packagesDialogValue4', 'Dark', callback);
    },
    setDisplayLanguageButton: function(callback) {
        return this.setValue('@displayDialogValue2', 'Japanese', callback);
    },
    clickDisplayResetButton: function(callback) {

        return this.click('@displayDialogResetBtn', callback);
    },
    clickVersionButton: function(callback) {
        this.api.pause(1000);
        return this.click('@versionBtn')
            .waitForElementVisible('@versionDialog', 5000, callback);
    },
    clickTutorialButton: function(callback) {
        this.api.pause(1000);
        return this.click('@tutorialBtn', callback);
    },
    clickTutorialDoneBtn: function(callback){
        return this.click('@tutorialDoneBtn', callback);
    },
    clickTutorialSkipBtn: function(){
        return this.click('@tutorialSkipBtn');
    },
    selectAsrif: function(callback) {
        this.api.pause(1000);
        return this.click('@nodeListContainer').click('@nodeHarkDef1', callback);
    },
    selectEffect: function (callback) {
        this.api.pause(1000);
        return this.click('@dspEffects', callback);
    },
    selectSpeechRecognizer: function (callback) {
        this.api.pause(1000);
        return this.click('@speechRecognitionClient', callback);
    },
    clickSaveButton: function(callback){
        this.api.pause(1000);
        return this.click('@saveBtn', callback);
    },
    setFileName: function (callback) {
        return this.setValue('@saveFileName', 'sample_package', callback);
    },
    saveFile: function(client, callback){
        var that = this;
        return this.click('@okSave', function () {
            //check if alert appeared
            client.getAlertText(function(alertText){
                that.api.pause(1000);
                if(alertText.value == 'Overwrite network?') {
                    //accept alert
                    client.acceptAlert(function () {
                        that.api.pause(1000);
                        if (callback) {
                            callback();
                        }
                    });
                }
                else {
                    if(callback) {
                        callback();
                    }
                }
            });
        });
    },
    clickFileManagerButton: function(callback){
        this.api.pause(1000);
        return this.click('@fileManagerBtn', callback);
    },
    loadFile: function(callback){
        this.api.pause(1000);
        return this.click('@loadTestFile', callback);
    },
    closeDialog: function(callback) {
        this.api.pause(1000);
        return this.click('@preferenceDialogClose', callback);
    },
    clickHarkDefBullet: function (callback) {
        return this.click('@enableHarkDefBtn', callback);
    },
    clickEnvironmentBtn: function (callback) {
        this.api.pause(1000);
        return this.click('@environmentBtn', callback);
    },
    clickTutorialNextBtn: function (callback) {
        this.api.pause(1000);
        return this.click('@tutorialNextBtn', callback);
    },
    selectSheet: function(selector, callback) {
        var that = this;
        return this.click(selector, function () {
            that.api.pause(1000, function () {
                if(callback) {
                    callback();
                }
            });
        });
    },
};

module.exports = {
    commands: [preferenceDialogueCommand],
    elements: {
        preferenceManagerBtn: xSelector('//*[@id="preferenceButton"]'),
        fileManagerBtn: xSelector('//*[@id="fileManagerButton"]'),
        saveBtn: xSelector('//*[@id="save"]'),
        dialogMain: xSelector('//*[@id="preferenceDialog"]/div[2]/div[1]'),
        preferenceDialogClose: xSelector('/html/body/div[@aria-describedby="preferenceDialog"]/div/button'),
        dataPath: xSelector('//*[@id="datapath"]'),
        filter: xSelector('//*[@id="fileManagerDialog"]/form/input'),
        fileList: xSelector('//*[@id="grid"]/table/tbody/tr'),
        pages: xSelector('//*[@id="paginator"]/div'),
        nodeListContainer: xSelector('//*[@id="nodelist"]'),
        saveFileName:xSelector('//*[@id="networkName"]'),
        okSave: xSelector('/html/body/div[@aria-describedby="saveFileDialog"]/div/div/button/span[text()="OK"]'),

        //packages dialog menu
        infoIcon: xSelector('//*[@id="packagesButton"]/span[2]'),
        packagesDialog: xSelector('//*[@id="preferenceDialog"]/div[@class="dialogMain"]/div[@class="package"]'),
        statusDialog: xSelector('//*[@id="console"]'),
        clearStatusButton: xSelector('//*[@id="clearExecuteStatus"]'),

        // packages
        package_nnet: xSelector('//div/div/div/div/form/fieldset/label[contains(text(), "NNet.def")]'),
        package_audioBlocks : xSelector('//div/div/div/div/form/fieldset/label[contains(text(), "audio_blocks.def")]'),
        package_dataflow: xSelector('//div/div/div/div/form/fieldset/label[contains(text(), "data-flow.def")]'),
        package_effects: xSelector('//div/div/div/div/form/fieldset/label[contains(text(), "effects.def")]'),
        package_fuzzy: xSelector('//div/div/div/div/form/fieldset/label[contains(text(), "fuzzy.def")]'),
        package_hark: xSelector('//div/div/div/div/form/fieldset/label[contains(text(), "hark.def")]'),
        package_hmm: xSelector('//div/div/div/div/form/fieldset/label[contains(text(), "hmm.def")]'),
        package_vq: xSelector('//div/div/div/div/form/fieldset/label[contains(text(), "vq.def")]'),

        // packages button
        packageBtn_nnet: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[1]/button/span[1]'),
        packageBtn_audioBlocks: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[2]/button/span[1]'),
        packageBtn_dataflow: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[3]/button/span[1]'),
        packageBtn_effects: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[4]/button/span[1]'),
        packageBtn_fuzzy: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[5]/button/span[1]'),
        packageBtn_hark: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[6]/button/span[1]'),
        packageBtn_hmm: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[7]/button/span[1]'),
        packageBtn_vq: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[8]/button/span[1]'),

        // packages theme
        packagesDialogValue1: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[1]/select'),
        packagesDialogValue2: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[2]/select'),
        packagesDialogValue3: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[3]/select'),
        packagesDialogValue4: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[4]/select'),
        packagesDialogValue5: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[5]/select'),
        packagesDialogValue6: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[6]/select'),
        packagesDialogValue7: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[7]/select'),
        packagesDialogValue8: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[8]/select'),

        //exec status
        execStatus: xSelector('//*[@id="executeStatus"]'),

        //canvas sheet
        canvasSheet: xSelector('//*[@id="canvas_MAIN"]'),

        //nodes
        nodeSpeechRecognitionClient: xSelector('//*[@id="nodelist"]/li[26]/ul/li[1]'),
        nodeEffectReverb: xSelector('//*[@id="node_Reverb_1"]'),
        nodeDspEffect: xSelector('//*[@id="nodelist"]/li[22]'),

        //display dialog menu
        displayDialogLine1: xSelector('//*[@id="preferenceDialog"]/div[2]/div[1]/ul/li/label[1]'),
        displayDialogLine2: xSelector('//*[@id="preferenceDialog"]/div[2]/div[1]/ul/li/label[2]'),
        displayDialogLine3: xSelector('//*[@id="preferenceDialog"]/div[2]/div[1]/ul/li/label[3]'),
        displayDialogLine4: xSelector('//*[@id="preferenceDialog"]/div[2]/div[1]/ul/li/label[4]'),
        displayDialogLine5: xSelector('//*[@id="preferenceDialog"]/div[2]/div[1]/ul/li/label[5]'),

        displayDialogValue1: xSelector('//*[@id="preferenceDialog"]/div[2]/div[1]/ul/li/select[1]'),
        displayDialogValue2: xSelector('//*[@id="preferenceDialog"]/div[2]/div[1]/ul/li/select[2]'),
        displayDialogValue3: xSelector('//*[@id="preferenceDialog"]/div[2]/div[1]/ul/li/select[3]'),
        displayDialogValue4: xSelector('//*[@id="preferenceDialog"]/div[2]/div[1]/ul/li/select[4]'),
        displayDialogValue5: xSelector('//*[@id="preferenceDialog"]/div[2]/div[1]/ul/li/select[5]'),
        displayDialogResetBtn: xSelector('//div/div/div/div/ul/li/button/span[text()="Reset"]'),

        //load file
        loadTestFile: xSelector('//div/div/div/div/table/tbody/tr/td[text()="sample_package"]/following-sibling::td/button/span[contains(@class, "ui-icon-folder-open")]'),
        samplePackageNetwork: xSelector('//div/div/div/div/table/tbody/tr/td[text()="sample_package"]'),
        //version dialog
        versionDialog: xSelector('//*[@id="preferenceDialog"]/div/div[@class="version"]'),

        //environment dialog
        environmentDialog: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]'),
        environmentLabel: xSelector('//*[@id="environmentDefault"]/label'),

        //help menu button
        helpButton: xSelector('//*[@id="helpButton"]'),

        //node list for hark library
        nodeHarkDef1: xSelector('//*[@id="nodelist"]/li[26]'),
        nodeHarkDef2: xSelector('//*[@id="nodelist"]/li[27]'),
        nodeHarkDef3: xSelector('//*[@id="nodelist"]/li[28]'),
        nodeHarkDef4: xSelector('//*[@id="nodelist"]/li[29]'),
        nodeHarkDef5: xSelector('//*[@id="nodelist"]/li[30]'),
        nodeHarkDef6: xSelector('//*[@id="nodelist"]/li[31]'),
        nodeHarkDef7: xSelector('//*[@id="nodelist"]/li[32]'),
        nodeHarkDef8: xSelector('//*[@id="nodelist"]/li[33]'),

        speechRecognitionClient: xSelector('//*[@id="nodelist"]/li[26]/ul/li[1]'),
        speechRecognitionSMNClient: xSelector('//*[@id="nodelist"]/li[26]/ul/li[2]'),

        dspEffects: xSelector('//li[contains(@key, "DSP:Effects")]'),
        reverb: xSelector('//li[contains(@key, "Reverb")]'),
        saturate: xSelector('//li[contains(@key, "Saturate")]'),

        fuzzy: xSelector('//li[contains(@key, "Fuzzy")]'),
        fuzzyModelExec: xSelector('//li[contains(@key, "FuzzyModelExec")]'),
        fuzzyRule: xSelector('//li[contains(@key, "FuzzyRule")]'),
        fuzzySet: xSelector('//li[contains(@key, "FuzzySet")]'),
        genericModel: xSelector('//li[contains(@key, "GenericModel")]'),
        inferenceModel: xSelector('//li[contains(@key, "InferenceModel")]'),
        trapezoidalFunction: xSelector('//li[contains(@key, "TrapezoidalFunction")]'),
        triangularFunction: xSelector('//li[contains(@key, "TriangularFunction")]'),

        enableHarkDefBtn: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[6]/button'),
        harkDefSelect: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/fieldset[6]/select'),

        environmentVariable: xSelector('//*[@id="preferenceDialog"]/div[2]/div[2]/form/button[contains(@class, "addvar")]'),

        preferenceDialog: xSelector('/html/body/div[contains(@aria-describedby, "preferenceDialog")]'),
        packagesBtn: xSelector('//div/div/div/ul/li/button/span[contains(text(), "Packages")]'),
        displayBtn: xSelector('//div/div/div/ul/li/button/span[contains(text(), "Display")]'),
        versionBtn: xSelector('//div/div/div/ul/li/button/span[contains(text(), "Version")]'),
        resetBtn: xSelector('//div/div/div/ul/li/button/span[contains(text(), "All Reset")]'),
        tutorialBtn: xSelector('//div/div/div/ul/li/button/span[contains(text(), "Tutorial")]'),
        environmentBtn: xSelector('//div/div/div/ul/li/button/span[contains(text(), "Environment")]'),
        addEnvironmentBtn: xSelector('//div/div/div/div/form/button/span[contains(text(), "Add new variable")]'),
        variableNameInput: xSelector('//*[@id="varname1"]'),
        variableValueInput: xSelector('//*[@id="varval1"]'),
        applyEnvVariable: xSelector('//div/div/div/div/form/button/span[contains(text(), "Apply")]'),
        closePreferencesDialog: xSelector('//div/div/button/span[contains(text(), "close")]'),

        tutorialLabelStep: xSelector('/html/body/div/div/div[contains(@class, "introjs-tooltiptext")]'),
        tutorialNextBtn: xSelector('/html/body/div/div/div/a[contains(text(), "Next →")]'),
        tutorialDoneBtn: xSelector('/html/body/div/div/div/a[contains(text(), "Done")]'),
        tutorialSkipBtn: xSelector('/html/body/div/div/div/a[contains(text(), "Skip")]'),
        tutorialLayer: xSelector('/html/body/div[contains(@class, "introjs-helperLayer")]'),

        uploadBtn: xSelector('//*[@id="fileUploadFile"]'),
        loadNetworkBtn: xSelector('//div/div/div/div/table/tbody/tr/td[text()="sample.n"]/following-sibling::td/button/span[contains(@class, "ui-icon-folder-open")]'),
        loopSheetTab: xSelector('//*[@id="sheets"]/ul/li[@aria-controls="sheet_LOOP"]')
    }
};